from django.db import models
from django.utils import translation
from django.conf import settings
from mptt.models import MPTTModel, TreeForeignKey
from garpix_page.models import BasePage


class MenuItem(MPTTModel):
    """
    Пункты меню.
    """
    title_for_admin = models.CharField(max_length=100, blank=True, default='', verbose_name='Название для админа')
    title = models.CharField(max_length=100, verbose_name='Название')
    menu_type = models.CharField(default='', max_length=100, choices=settings.CHOICE_MENU_TYPES, verbose_name='Тип меню')
    page = models.ForeignKey(BasePage, null=True, blank=True, verbose_name='Страница, на которую ведет пункт меню', help_text='Если этот пункт не выбран, то будет использовано следующее поле "Внешний URL"', on_delete=models.CASCADE)
    url = models.CharField(max_length=1000, null=True, blank=True, verbose_name='Внешний URL')
    is_active = models.BooleanField(default=True, verbose_name='Включено')
    target_blank = models.BooleanField(default=False, verbose_name='Открывать в новом окне')
    created_at = models.DateTimeField(auto_now_add=True, verbose_name='Дата создания')
    updated_at = models.DateTimeField(auto_now=True, verbose_name='Дата изменения')
    sort = models.IntegerField(default=100, verbose_name='Сортировка', help_text='Чем меньше число, тем выше будет элемент в списке.')
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children', db_index=True, verbose_name='Родительский пункт меню', on_delete=models.CASCADE)
    is_current = False
    is_current_full = False

    class Meta:
        verbose_name = 'Пункт меню'
        verbose_name_plural = 'Пункты меню'
        ordering = ('sort', )

    def __str__(self):
        if self.title_for_admin:
            return self.title_for_admin
        return self.title

    def get_link(self):
        if self.page:
            return self.page.get_absolute_url()
        elif self.url:
            if self.url.startswith('/'):
                current_language_code_url_prefix = translation.get_language()
                if current_language_code_url_prefix == settings.LANGUAGE_CODE:
                    return "{}".format(self.url)
                return "/{}{}".format(current_language_code_url_prefix, self.url)
            else:
                return self.url
        else:
            return '#'
    get_link.short_description = 'URL'

    def get_menu_params(self):
        return settings.MENU_TYPES[self.menu_type]



from modeltranslation.translator import TranslationOptions, register
from ..models import MenuItem


@register(MenuItem)
class MenuItemTranslationOptions(TranslationOptions):
    fields = ('title', )



from django.contrib import admin
from django.db import models
from django.forms import CheckboxSelectMultiple
from mptt.admin import DraggableMPTTAdmin
from garpix_menu.models import MenuItem
from modeltranslation.admin import TabbedTranslationAdmin


@admin.register(MenuItem)
class MenuItemAdmin(TabbedTranslationAdmin, DraggableMPTTAdmin):
    actions = ('rebuild',)
    list_filter = ('parent', 'is_active')
    list_display = ('tree_actions', 'indented_title', 'title', 'menu_type', 'get_link', 'target_blank', 'is_active', 'sort')
    formfield_overrides = {
        models.ManyToManyField: {'widget': CheckboxSelectMultiple},
    }
    search_fields = ('title',)
    list_editable = ('sort',)
    list_display_links = ('indented_title',)

    def _rebuild(self):
        try:
            self.model.objects.rebuild()
        except:  # noqa
            pass

    def rebuild(self, request, queryset):
        """
        Пересорбать МПТТ модель. Иногда требуется для перезагрузки дерева.
        """
        self._rebuild()
    rebuild.short_description = 'Пересобрать пункты раздела'

    def save_model(self, request, obj, form, change):
        # пересобрать МПТТ объекты
        self._rebuild()
        super().save_model(request, obj, form, change)



"""
Контекстный процессор для меню.
"""
from django.conf import settings
from garpix_menu.models import MenuItem


def check_is_home(url):
    urls = ['', '/']
    for item in settings.LANGUAGES:
        urls.append(f'/{item[0]}')
    return url in urls


def menu_processor(request):
    """
    Контекстный процессор для возможности отображения всех меню на сайте.
    Меню обычно распологаются на нескольких страницах, поэтому вынесено сюда.
    """
    current_path = request.path
    current_path_without_slash = current_path
    if current_path_without_slash[-1] == '/':
        current_path_without_slash = current_path_without_slash[0:-1]

    context = {
        'menus': {}
    }
    for menu_type_arr in settings.CHOICE_MENU_TYPES:
        menu = MenuItem.objects.filter(is_active=True, menu_type=menu_type_arr[0], parent=None)
        context['menus'][menu_type_arr[0]] = menu.order_by('sort', 'title')
        for menu_item in context['menus'][menu_type_arr[0]]:
            if menu_item.get_link() in (current_path, current_path_without_slash):
                menu_item.is_current = True
                menu_item.is_current_full = True
            elif current_path.startswith(menu_item.get_link()):
                if not check_is_home(menu_item.get_link()):
                    menu_item.is_current = True
            elif menu_item.url and menu_item.url.endswith(current_path):
                if not check_is_home(menu_item.get_link()):
                    menu_item.is_current = True
    return context
